"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var sl_js_infra_1 = require("sl-js-infra");
var getSystemDateValueOf = sl_js_infra_1.SlJsInfra.getSystemDateValueOf;
var LogsProcess = /** @class */ (function () {
    function LogsProcess(cfg, bufferingLogger, logServiceProxy, 
    /**
     * Environments service, used to get environmental data for submission (process id, machine data, etc.)
     */
    environmentDataService, agentDataService, 
    /**
     * Background thread that tells the events process to send events from the queue to the server
     */
    sendToServerWatchdog, 
    /**
     * Foreground thread that prevents the process from shutting down, and kept alive while there are items to submit to the server or items are currently being submitted
     */
    keepaliveWatchdog, logger) {
        var _this = this;
        this.cfg = cfg;
        this.bufferingLogger = bufferingLogger;
        this.logServiceProxy = logServiceProxy;
        this.environmentDataService = environmentDataService;
        this.agentDataService = agentDataService;
        this.sendToServerWatchdog = sendToServerWatchdog;
        this.keepaliveWatchdog = keepaliveWatchdog;
        this.logger = logger;
        this.sequence = 0;
        this.isSubmittingLogs = false;
        this.isRunning = false;
        this.stopCallbacks = [];
        if (!cfg) {
            throw new Error("cfg is required");
        }
        if (!bufferingLogger) {
            throw new Error("bufferingLogger is required");
        }
        if (!logServiceProxy) {
            throw new Error("logServiceProxy is required");
        }
        if (!environmentDataService) {
            throw new Error("environmentDataService is required");
        }
        if (!agentDataService) {
            throw new Error("agentDataService is required");
        }
        if (!sendToServerWatchdog) {
            throw new Error("sendToServerWatchdog was not specified");
        }
        if (!keepaliveWatchdog) {
            throw new Error("keepaliveWatchdog was not specified");
        }
        bufferingLogger.on("log_buffer_full", function (buffer) {
            if (buffer.length) {
                _this.submitLogBuffer(buffer);
            }
        });
        var context = this;
        sendToServerWatchdog.on("alarm", function () {
            if (_this.cfg.sendLogs.value)
                _this.bufferingLogger.checkBuffer(true);
        });
    }
    LogsProcess.prototype.createLogsPacket = function (items) {
        var packet = {
            customerId: this.cfg.customerId.value,
            appName: this.cfg.appName.value,
            environment: this.environmentDataService.getEnvDataForSubmission(),
            configurationData: this.cfg.toJsonObject(),
            logs: items.map(function (t) { return JSON.stringify(t); }),
            meta: {
                sequence: ++this.sequence,
                generated: getSystemDateValueOf(),
                agentId: this.agentDataService.getAgentId(),
            },
        };
        return packet;
    };
    LogsProcess.prototype.submitLogBuffer = function (items) {
        var _this = this;
        if (!items.length)
            return;
        var packet = this.createLogsPacket(items);
        if (this.isSubmittingLogs) {
            //drop logs packet
            return;
        }
        this.isSubmittingLogs = true;
        this.logServiceProxy.submitLogs(packet, function (err) {
            if (err) {
                //log?
            }
            _this.isSubmittingLogs = false;
            _this.callStopCallbacks();
        });
    };
    LogsProcess.prototype.start = function () {
        this.isRunning = true;
        this.sendToServerWatchdog.start();
    };
    LogsProcess.prototype.callStopCallbacks = function () {
        var _this = this;
        if (this.stopCallbacks.length > 0) {
            this.stopCallbacks.forEach(function (cb) {
                try {
                    cb();
                }
                catch (err) {
                    _this.environmentDataService.setLastError(err.stack);
                }
            });
        }
    };
    LogsProcess.prototype.stop = function (callback) {
        var _this = this;
        this.sendToServerWatchdog.stop();
        try {
            if (this.cfg.enabled.value === false || this.cfg.sendLogs.value === false) { //Log sending/Agent functionality is disabled. Return immediately
                this.isRunning = false;
                return callback();
            }
            this.bufferingLogger.checkBuffer(true);
            this.bufferingLogger.stop();
            if (this.isSubmittingLogs) {
                this.stopCallbacks.push(function () {
                    _this.isRunning = false;
                    _this.keepaliveWatchdog.stop();
                    return callback();
                });
            }
            else {
                this.isRunning = false;
                this.keepaliveWatchdog.stop();
                return callback();
            }
        }
        catch (err) {
            return callback();
        }
    };
    return LogsProcess;
}());
exports.LogsProcess = LogsProcess;
//# sourceMappingURL=index.js.map